
// Importovanie všetkých tried z balíčka knižnica.
import knižnica.*;

// Začiatok definície triedy odvodenej
// od nadradenej triedy GRobot.
public class Asteroid extends GRobot
{
	// Definovanie oblasti na kreslenie tvaru:
	private final Oblasť tvar = new Oblasť();

	// Konštruktor nastavujúci niektoré vlastnosti
	// (konkrétne jeho veľkosť a smer na objekt).
	public Asteroid(double veľkosť, Poloha objekt)
	{
		// Volanie predvoleného konštruktora:
		this();

		upravVeľkosť(veľkosť);
		náhodnáPolohaNaOkraji();
		nasmerujNa(objekt);
	}

	// Predvolený konštruktor.
	public Asteroid()
	{
		zdvihniPero();
		hrúbkaČiary(1);
		ohranič();
		skry();
	}

	// Keď sa asteroid rozbije, tak vznikne jeho
	// „dvojička“. Touto metódou jej nastavíme
	// podobné vlastnosti ako tomuto (zmenšenému)
	// asteroidu.
	public void aktivujDvojičku(Asteroid dvojička)
	{
		dvojička.upravVeľkosť(veľkosť());
		dvojička.skočNa(this);
		dvojička.nasmerujNáhodne();
		dvojička.aktivuj();
	}

	// Pregenerovanie polygónu asteroidu.
	public void upravVeľkosť(double veľkosť)
	{
		// Musíme upraviť veľkosť asteroidu,
		// aby boli správne vyhodnotené kolízie:
		veľkosť(veľkosť);
		mierka(1);

		// Do tohto poľa budeme ukladať body
		// náhodne generovaného polygónu:
		Bod[] body = new Bod[10];

		// Toto je záloha aktuálnej polohy asteroidu:
		Bod poloha = poloha();

		// Vygenerujeme 10 bodov na obvode kružnice
		// lemujúcej asteroid:
		for (int i = 0; i < 10; ++i)
		{
			skočNa(0, 0);
			smer(i * 36);
			skoč(veľkosť * Svet.
				náhodnéReálneČíslo(1, 2));
			body[i] = poloha();
		}

		// Z bodov vytvoríme cestu
		// a tú potom pridáme do oblasti:
		skočNa(body[0]); začniCestu();
		for (int i = 1; i < 10; ++i) skočNa(body[i]);
		tvar.vymaž(); tvar.pridaj(cesta());

		// Vrátim sa na pôvodnú polohu:
		skočNa(poloha);

		// Asteroid sa bude pomaly automaticky otáčať:
		otáčajTvar(Svet.
			náhodnéCeléČíslo(-8, 8), false);
	}

	// Túto metódu použijeme na počiatočné
	// umiestnenie asteroidu:
	public void náhodnáPolohaNaOkraji()
	{
		skočNa(0, 0); náhodnýSmer();
		skoč(Svet.náhodnéCeléČíslo(300, 310));
	}

	// Touto metódou nasmerujeme asteroid na určený
	// objekt s náhodnou odchýlkou:
	public void nasmerujNa(Poloha objekt)
	{
		otočNa(objekt);
		otočO(Svet.náhodnéReálneČíslo(-15, 15));
		rýchlosť(Svet.náhodnéReálneČíslo(2, 3.5),
			false);
	}

	// Touto metódou nasmerujeme asteroid náhodným
	// smerom (to bude užitočné pri rozbíjaní
	// asteroidov):
	public void nasmerujNáhodne()
	{
		náhodnýSmer();
		rýchlosť(Svet.náhodnéReálneČíslo(2, 3.5),
			false);
	}
	
	// Vlastné kreslenie tvaru.
	@Override public void kresliTvar()
	{
		obkresliOblasť(tvar);
		double veľkosť = veľkosť();
		veľkosť(veľkosť() - 2);
		obkresliOblasť(tvar);
		veľkosť(veľkosť);
	}

	// Reakcia na aktiváciu.
	@Override public void aktivácia()
	{
		zobraz();
	}

	// Reakcia na deaktiváciu.
	@Override public void deaktivácia()
	{
		skry();
	}
}
